﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.MapSuiteGeocoder;
using System.IO;

namespace ThinkGeoGIS_PoC.Forms
{
    public partial class AddressSelectionDialog : Form
    {

        #region Private Instance Fields

        private string mapbase = Path.Combine(Environment.CurrentDirectory, "MapData");
        private string _GISLocation = string.Empty;

        #endregion

        #region Constructors

        public AddressSelectionDialog()
        {
            InitializeComponent();
            this.Load += new EventHandler(AddressSelectionDialog_Load);
            this.btnMap.Click += new EventHandler(btnMap_Click);
            this.btnDone.Click += new EventHandler(btnDone_Click);
            this.lstMatches.DoubleClick += new EventHandler(lstMatches_DoubleClick);
        }

        protected void lstMatches_DoubleClick(object sender, EventArgs e)
        {
            SetLocation();
            btnDone_Click(this, new EventArgs());
        }


        #endregion

        #region Events

        protected void AddressSelectionDialog_Load(object sender, EventArgs e)
        {
            GeocodeLocation();
        }

        protected void btnMap_Click(object sender, EventArgs e)
        {

        }

        protected void btnDone_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
        }

        #endregion

        #region Public Instance Properties

        public string County { get; set; }
        public string Location { get; set; }

        #endregion

        #region Public Instance Methods



        #endregion

        #region Private Instance Methods

        private void GeocodeLocation()
        {
            UsaGeocoder usaGeocoder = new UsaGeocoder(Path.Combine(mapbase, "ChicagoData"), MatchStatus.FuzzyMatch);
            MatchResult matchResult = new MatchResult();
            try
            {
                usaGeocoder.Open();
                matchResult = usaGeocoder.Match(String.Format("{0} {1} county IN", Location, County));
            }
            catch (Exception ex)
            {
                MessageBox.Show(String.Format("RevGeocode error:\n{0}", ex.ToString()));
            }
            finally
            {
                usaGeocoder.Close();
            }

            PopulateAddressResultList(matchResult);
        }

        private void PopulateAddressResultList(MatchResult matchResult)
        {
            if (matchResult.MatchItems.Count == 0)
            {
                // Force verify
                btnMap_Click(this, new EventArgs());
                return;
            }

            // Load the matching items into the grid
            foreach (MatchItem matchItem in matchResult.MatchItems)
            {
                lstMatches.Items.Add(new ListItem(matchItem, new string[] { "HouseNumber", "PrefixDirectional", "Street", "StreetType", "SuffixDirectional", "City" } ));
            }
        }

        private void SetLocation()
        {
            if (lstMatches.Items.Count > 0 && String.IsNullOrEmpty(_GISLocation))
            {
                _GISLocation = (string)lstMatches.SelectedValue;
            }

            Location = _GISLocation;

        }

        #endregion
        
    }
}
