﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.MapSuiteGeocoder;
using System.IO;

namespace ThinkGeoGIS_PoC.Forms
{
    public partial class EnterLocation : Form
    {

        #region Private Instance Fields

        private string mapbase = Path.Combine(Environment.CurrentDirectory, "MapData");
        private string INmapbase = @"C:\Indiana Map Data\IN";
        private string GeocoderPath = @"C:\ThinkGeo\GeocodeData\Map Suite 3.0 Geocoder Index";
        private UsaGeocoder _usaGeocoder = null;
        private List<Address> _Locations = null;
        private AutoCompleteStringCollection _AutoCompleteLocations = null;
            
        #endregion

        #region Constructors

        public EnterLocation()
        {
            InitializeComponent();
            this.Load += new EventHandler(EnterLocation_Load);
            this.txtLocation.LostFocus += new EventHandler(txtLocation_LostFocus);
            this.btnMap.Click += new EventHandler(btnMap_Click);
            this.txtLocation.TextChanged += new EventHandler(txtLocation_TextChanged);
            this.FormClosing += new FormClosingEventHandler(EnterLocation_FormClosing);
        }

        

        #endregion

        #region Events

        protected void txtLocation_TextChanged(object sender, EventArgs e)
        {
            
            MatchResult matchResult = new MatchResult();
            try
            {
                usaGeocoder.Open();
                matchResult = usaGeocoder.Match(String.Format("{0} {1} county IN", txtLocation.Text, txtCounty.Text));
            }
            catch (Exception ex)
            {
                MessageBox.Show(String.Format("Geocode error:\n{0}", ex.ToString()));
            }
            finally
            {
                usaGeocoder.Close();
            }

            FillAutoCompleteList(matchResult);
        }

        protected void EnterLocation_FormClosing(object sender, FormClosingEventArgs e)
        {
            //try
            //{
            //    usaGeocoder.Close();
            //}
            //catch { }
        }

        protected void EnterLocation_Load(object sender, EventArgs e)
        {
            txtLocation.Focus();

            pbVerified.Visible = false;
            //try
            //{
            //    usaGeocoder.Open();
            //}
            //catch { }
            
            
        }

        protected void txtLocation_LostFocus(object sender, EventArgs e)
        {
            Address temp = Locations.Where(l => l.AddressText == txtLocation.Text).FirstOrDefault();
            if (temp != null)
            {
                pbVerified.Visible = true;
                FinalLocation = temp;
            }
            else
                pbVerified.Visible = false;
        }

        protected void btnMap_Click(object sender, EventArgs e)
        {
            using (VerifyMap vm = new VerifyMap())
            {
                vm.CountyName = txtCounty.Text;
                vm.UserEnteredLocation = txtLocation.Text;
                vm.ShowDialog();
                txtLocation.Text = vm.UserEnteredLocation;
                pbVerified.Visible = vm.Verified;
            }
        }

        #endregion

        #region Private Instance Properties

        private Address FinalLocation { get; set; }

        private UsaGeocoder usaGeocoder
        {
            get
            {
                if (_usaGeocoder == null)
                    _usaGeocoder = new UsaGeocoder(GeocoderPath, MatchStatus.FuzzyMatch);
                return _usaGeocoder;
            }
        }

        private List<Address> Locations
        {
            get
            {
                if (_Locations == null)
                    _Locations = new List<Address>();
                return _Locations;
            }
            set
            {
                _Locations = value;
            }
        }

        private AutoCompleteStringCollection AutoCompleteLocations
        {
            get
            {
                if (_AutoCompleteLocations == null)
                    _AutoCompleteLocations = new AutoCompleteStringCollection();
                return _AutoCompleteLocations;
            }
            set
            {
                _AutoCompleteLocations = value;
            }
        }

        #endregion

        #region Private Instance Methods

        private void FillAutoCompleteList(MatchResult matchResult)
        {
            if (matchResult.MatchItems.Count < 1)
            {
                return;
            }
            
            foreach (MatchItem matchItem in matchResult.MatchItems)
            {
                Address temp = new Address(matchItem, new string[] { "HouseNumber", "PrefixDirectional", "Street", "StreetType", "SuffixDirectional", "City" });
                if (!String.IsNullOrEmpty(temp.AddressText))
                {
                    if (!Locations.Contains(temp))
                        Locations.Add(temp);
                    if (!AutoCompleteLocations.Contains(temp.AddressText))
                    {
                        Console.WriteLine(temp.AddressText);
                        AutoCompleteLocations.Add(temp.AddressText);
                    }
                }
            }
            txtLocation.AutoCompleteCustomSource = AutoCompleteLocations;
        }

        #endregion

    }
}
