﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.DesktopEdition;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.MapSuiteGeocoder;
using System.IO;
using System.Collections.ObjectModel;

namespace ThinkGeoGIS_PoC.Forms
{
    public partial class IntersectionTesting : Form
    {
        #region Private Instance Fields

        private string INmapbase = @"C:\Indiana Map Data\IN";
        private string mapbase = Path.Combine(Environment.CurrentDirectory, "MapData");

        #endregion

        #region Constructors

        public IntersectionTesting()
        {
            InitializeComponent();

            this.Load += new EventHandler(IntersectionTesting_Load);
            this.btnZoomIn.Click += new EventHandler(btnZoomIn_Click);
            this.btnZoomOut.Click += new EventHandler(btnZoomOut_Click);
            this.btnSearch.Click += new EventHandler(btnSearch_Click);
            this.btnComboSearch.Click += new EventHandler(btnComboSearch_Click);
        }

        

        #endregion

        #region Events
        
        protected void btnComboSearch_Click(object sender, EventArgs e)
        {
            string[] streets = txtCombo.Text.Split('&');
            string street1 = streets[0].Trim();
            string street2 = string.Empty;
            if (streets.Count() > 1)
                street2 = streets[1].Trim();

            LayerOverlay StaticStreetOverlay = wfmapMap.Overlays["StaticStreetOverlay"] as LayerOverlay;
            ShapeFileFeatureLayer streetLayer = StaticStreetOverlay.Layers["StreetLayer"] as ShapeFileFeatureLayer;
            Collection<Feature> features = streetLayer.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns);
            List<Feature> extentFeatures = new List<Feature>();

            extentFeatures.AddRange(MapStreet(features, "intersections", "Street", "NAME", street1));
            if (!String.IsNullOrEmpty(street2))
                extentFeatures.AddRange(MapStreet(features, "intersections", "Street2", "NAME", street2));

            wfmapMap.CurrentExtent = ExtentHelper.GetBoundingBoxOfItems(extentFeatures);
            wfmapMap.Refresh();
        }

        protected void btnSearch_Click(object sender, EventArgs e)
        {
            

            LayerOverlay StaticStreetOverlay = wfmapMap.Overlays["StaticStreetOverlay"] as LayerOverlay;
            ShapeFileFeatureLayer streetLayer = StaticStreetOverlay.Layers["StreetLayer"] as ShapeFileFeatureLayer;
            streetLayer.Open();
            //Collection<Feature> features = streetLayer.FeatureSource.GetFeaturesByColumnValue("RECID", "12503");
            //Collection<Feature> features = streetLayer.FeatureSource.GetFeaturesByColumnValue("NAME", "12th  St");
            Collection<Feature> features = streetLayer.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns);
            Collection<Feature> streetfeatures = new Collection<Feature>();
            streetLayer.Close();
            

            LayerOverlay intersectionOverlay = wfmapMap.Overlays["intersections"] as LayerOverlay;
            
            // Street 1 ---------------

            List<MultilineShape> streetShapes = new List<MultilineShape>();
            foreach (Feature f in features.Where(r => r.ColumnValues["FENAME"].ToUpper().Contains(txtStreet1.Text.ToUpper())))
            {
                streetShapes.Add(f.GetShape() as MultilineShape);
                streetfeatures.Add(f);
            }

            InMemoryFeatureLayer streetInMemoryFeatureLayer = intersectionOverlay.Layers["Street"] as InMemoryFeatureLayer;
            streetInMemoryFeatureLayer.Open();
            streetInMemoryFeatureLayer.InternalFeatures.Clear();
            streetInMemoryFeatureLayer.EditTools.BeginTransaction();
            foreach (MultilineShape m in streetShapes)
            {
                streetInMemoryFeatureLayer.EditTools.Add(new Feature(m));
                streetInMemoryFeatureLayer.EditTools.Add(new Feature(GetEndPoints(m)));
            }
            streetInMemoryFeatureLayer.EditTools.CommitTransaction();
            streetInMemoryFeatureLayer.Close();
            // ----------------------

            // Street 2 ---------------
            List<MultilineShape> streetShapes2 = new List<MultilineShape>();
            foreach (Feature f in features.Where(r => r.ColumnValues["FENAME"].ToUpper().Contains(txtStreet2.Text.ToUpper())))
            {
                streetShapes2.Add(f.GetShape() as MultilineShape);
                streetfeatures.Add(f);
            }

            InMemoryFeatureLayer streetInMemoryFeatureLayer2 = intersectionOverlay.Layers["Street2"] as InMemoryFeatureLayer;
            streetInMemoryFeatureLayer2.Open();
            streetInMemoryFeatureLayer2.InternalFeatures.Clear();
            streetInMemoryFeatureLayer2.EditTools.BeginTransaction();
            foreach (MultilineShape m in streetShapes2)
            {
                streetInMemoryFeatureLayer2.EditTools.Add(new Feature(m));
                streetInMemoryFeatureLayer2.EditTools.Add(new Feature(GetEndPoints(m)));
            }
            streetInMemoryFeatureLayer2.EditTools.CommitTransaction();
            streetInMemoryFeatureLayer2.Close();
            // ----------------------
            

            wfmapMap.CurrentExtent = ExtentHelper.GetBoundingBoxOfItems(streetfeatures);
            wfmapMap.Refresh();
        }

        protected void btnZoomOut_Click(object sender, EventArgs e)
        {
            wfmapMap.ZoomOut(20);
            wfmapMap.Refresh();
        }

        protected void btnZoomIn_Click(object sender, EventArgs e)
        {
            wfmapMap.ZoomIn(60);
            wfmapMap.Refresh();
        }

        protected void IntersectionTesting_Load(object sender, EventArgs e)
        {
            lblX.Text = lblY.Text = string.Empty;

            wfmapMap.MapUnit = GeographyUnit.DecimalDegree;

            //wfmapMap.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean);
            wfmapMap.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.Grass);

            InMemoryFeatureLayer pointLayer = new InMemoryFeatureLayer();
            pointLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.PointType = PointType.Bitmap;
            pointLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.Image = new GeoImage(Path.Combine(mapbase, "Semi Truck.png"));
            pointLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            WorldMapKitWmsDesktopOverlay wmkOverlay = new WorldMapKitWmsDesktopOverlay();
            //wfmapMap.Overlays.Add("WorldMapKitWmsOverlay", wmkOverlay);

            // Austin Streets Layer --------
            //ShapeFileFeatureLayer streetLayer = new ShapeFileFeatureLayer(Path.Combine(mapbase, "austinstreets.shp"));
            ShapeFileFeatureLayer streetLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039lkA.shp"));
            streetLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.LocalRoad1;
            streetLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay StaticStreetOverlay = new LayerOverlay();
            StaticStreetOverlay.Layers.Add("StreetLayer", streetLayer);
            wfmapMap.Overlays.Add("StaticStreetOverlay", StaticStreetOverlay);
            // ----------------------------

            // Road lines layer  First Street-----------
            InMemoryFeatureLayer streetInMemoryFeatureLayer = new InMemoryFeatureLayer();
            streetInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.LightPink, 3, true);
            streetInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Circle, GeoColor.StandardColors.Red, 12);
            streetInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            // ----------------------------

            LayerOverlay pointOverlay = new LayerOverlay();
            pointOverlay.Layers.Add("PointLayer", pointLayer);
            wfmapMap.Overlays.Add("PointOverlay", pointOverlay);
            

            // Road lines layer  Second Street-----------
            InMemoryFeatureLayer streetInMemoryFeatureLayer2 = new InMemoryFeatureLayer();
            streetInMemoryFeatureLayer2.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.LightBlue, 3, true);
            streetInMemoryFeatureLayer2.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Circle, GeoColor.StandardColors.Blue, 12);
            streetInMemoryFeatureLayer2.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            // ----------------------------

            LayerOverlay intersectionOverlay = new LayerOverlay();
            intersectionOverlay.Layers.Add("Street", streetInMemoryFeatureLayer);
            intersectionOverlay.Layers.Add("Street2", streetInMemoryFeatureLayer2);
            wfmapMap.Overlays.Add("intersections", intersectionOverlay);

            // Marker Overlay
            LayerOverlay markerOverlay = new LayerOverlay();
            wfmapMap.Overlays.Add("MarkerOverlay", markerOverlay);

            // Marker layer
            MarkerLayer markerLayer = new MarkerLayer();
            markerOverlay.Layers.Add("MarkerLayer", markerLayer);

            ScaleLineAdornmentLayer scaleLine = new ScaleLineAdornmentLayer();
            scaleLine.Location = AdornmentLocation.LowerLeft;
            scaleLine.XOffsetInPixel = 25f;
            scaleLine.YOffsetInPixel = -25f;

            wfmapMap.AdornmentOverlay.Layers.Add("ScaleLineAdornmentLayer", scaleLine);

            // ------------------------------------------
            // Test to home in on zip on load
            //bool FilterWorked = false;
            //if (!String.IsNullOrEmpty(CountyFilter))
            //{
            //    // Get the path to the data files and create the Geocoder
            //    string dataPath = Path.Combine(mapbase, "ChicagoData");
            //    UsaGeocoder usaGeocoder = new UsaGeocoder(dataPath, MatchStatus.FuzzyMatch);

            //    // Open the geocoder, get any matches and close it
            //    MatchResult matchResult;
            //    try
            //    {
            //        usaGeocoder.Open();
            //        matchResult = usaGeocoder.Match(String.Format("{0} county IN", CountyFilter));
            //    }
            //    finally
            //    {
            //        usaGeocoder.Close();
            //    }
            //    if (matchResult.MatchItems.Count > 0)
            //    {
            //        FilterWorked = true;
            //        MatchItem matchItem = matchResult.MatchItems[0];
            //        wfmapMap.CurrentExtent = new RectangleShape(matchItem.MatchPairs["BoundingBox"]);
            //    }
            //}
            //if (!FilterWorked)
            //    wfmapMap.CurrentExtent = new RectangleShape(-88.3330001640625, 42.5966329101563, -86.9157638359375, 41.1629170898438);

            // ------------------------------------------

            //wfmapMap.CurrentExtent = new RectangleShape(-97.8101, 30.3618, -97.6630, 30.2213);
            streetLayer.Open();
            wfmapMap.CurrentExtent = streetLayer.GetBoundingBox();
            streetLayer.Close();
            //wfmapMap.MapClick += new EventHandler<MapClickWinformsMapEventArgs>(wfmapMap_MapClick);            

            wfmapMap.Refresh();
        }

        protected void wfmapMap_MapClick(object sender, MapClickWinformsMapEventArgs e)
        {
            //MessageBox.Show(string.Format("X:{0}\nY:{1}", e.WorldX.ToString("N4"), e.WorldY.ToString("N4")));
            lblX.Text = string.Format("Lon: {0}", e.WorldX.ToString("N4"));
            lblY.Text = string.Format("Lat: {0}", e.WorldY.ToString("N4"));

            Feature SelectedPoint = new Feature(e.WorldX, e.WorldY, "Point1");

            InMemoryFeatureLayer pointLayer = (InMemoryFeatureLayer)wfmapMap.FindFeatureLayer("PointLayer");
            if (!pointLayer.InternalFeatures.Contains("Point1"))
            {
                pointLayer.InternalFeatures.Add("Point1", SelectedPoint);
            }
            else
            {
                pointLayer.InternalFeatures.Remove("Point1");
                pointLayer.InternalFeatures.Add("Point1", SelectedPoint);
            }

            // Reverse Geocode Location
            //UsaGeocoder usaGeocoder = new UsaGeocoder(Path.Combine(mapbase, "ChicagoData"));
            UsaGeocoder usaGeocoder = new UsaGeocoder(mapbase);

            MatchResult matchResult = new MatchResult();
            try
            {
                usaGeocoder.Open();
                matchResult = usaGeocoder.Match(String.Format("{0} {1}", e.WorldX.ToString(), e.WorldY.ToString()));
            }
            catch (Exception ex)
            {
                MessageBox.Show(String.Format("RevGeocode error:\n{0}", ex.ToString()));
            }
            finally
            {
                usaGeocoder.Close();
            }

            if (matchResult.MatchItems.Count > 0)
            {
                MatchItem matchItem = matchResult.MatchItems[0];
                txtCombo.Text =
                    String.Format("{0} {1} {2} {3}, {4}, {5} County, {6}, {7}",
                    matchItem.MatchPairs["HouseNumber"],
                    matchItem.MatchPairs["PrefixDirectional"],
                    matchItem.MatchPairs["SuffixDirectional"],
                    matchItem.MatchPairs["Street"],
                    matchItem.MatchPairs["StreetType"],
                    matchItem.MatchPairs["City"],
                    matchItem.MatchPairs["County"],
                    matchItem.MatchPairs["State"],
                    matchItem.MatchPairs["Zip"]);
            }
            else
            {
                txtCombo.Text = "Could not find.";
            }

            wfmapMap.Refresh(wfmapMap.Overlays["PointOverlay"]);

        }

        #endregion

        #region Private Instance Methods

        private MultipointShape GetEndPoints(MultilineShape multilineShape)
        {
            MultipointShape multipointShape = new MultipointShape();

            foreach (LineShape lineShape in multilineShape.Lines)
            {
                multipointShape.Points.Add(new PointShape(lineShape.Vertices[0]));
                multipointShape.Points.Add(new PointShape(lineShape.Vertices[lineShape.Vertices.Count - 1]));
            }
            return multipointShape;
        }

        private List<Feature> MapStreet(Collection<Feature> features, string OverlayKey, string FeatureLayerKey, string ColumnName, string Value)
        {
            if (features == null || String.IsNullOrEmpty(OverlayKey) || String.IsNullOrEmpty(FeatureLayerKey) || String.IsNullOrEmpty(ColumnName) || String.IsNullOrEmpty(Value))
                return null;

            List<Feature> streetFeatures = new List<Feature>();
            LayerOverlay intersectionOverlay = wfmapMap.Overlays[OverlayKey] as LayerOverlay;
            
            // Street Drawing ---------------

            List<MultilineShape> streetShapes = new List<MultilineShape>();
            foreach (Feature f in features.Where(r => r.ColumnValues[ColumnName].ToUpper().Contains(Value.ToUpper())))
            {
                streetShapes.Add(f.GetShape() as MultilineShape);
                streetFeatures.Add(f);
            }

            InMemoryFeatureLayer streetInMemoryFeatureLayer = intersectionOverlay.Layers[FeatureLayerKey] as InMemoryFeatureLayer;
            streetInMemoryFeatureLayer.Open();
            streetInMemoryFeatureLayer.InternalFeatures.Clear();
            streetInMemoryFeatureLayer.EditTools.BeginTransaction();
            foreach (MultilineShape m in streetShapes)
            {
                streetInMemoryFeatureLayer.EditTools.Add(new Feature(m));
                streetInMemoryFeatureLayer.EditTools.Add(new Feature(GetEndPoints(m)));
            }
            streetInMemoryFeatureLayer.EditTools.CommitTransaction();
            streetInMemoryFeatureLayer.Close();
            // ----------------------

            return streetFeatures;
        }

        #endregion

        #region Public Instance Properties

        public string CountyFilter { get; set; }

        #endregion

    }
}
