﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using ThinkGeo.MapSuite.MapSuiteGeocoder;
using System.IO;
using System.Collections.ObjectModel;
using System.Text.RegularExpressions;

namespace ThinkGeoGIS_PoC.Forms
{
    public partial class VerifyMap : Form
    {
        

        #region Private Instance Fields

        private string mapbase = Path.Combine(Environment.CurrentDirectory, "MapData");
        private string INmapbase = @"C:\Indiana Map Data\IN";
        private string GeocoderPath = @"C:\ThinkGeo\GeocodeData\Map Suite 3.0 Geocoder Index";
        private UsaGeocoder _VerifyGeocoder = null;
        private List<Address> _Locations = null;
        private AutoCompleteStringCollection _AutoCompleteLocations = null;
        private List<string> _CountyZips = null;
        private int ClickCounter = 0;

        #endregion

        #region Constructors

        public VerifyMap()
        {
            InitializeComponent();
            this.Load += new EventHandler(VerifyMap_Load);
            //this.txtLocation.TextChanged += new EventHandler(txtLocation_TextChanged); // Commented out until issue with matching not being limited to Zips is resolved.
            this.txtLocation.KeyDown += new KeyEventHandler(txtLocation_KeyDown);
            this.mapMap.MapClick += new EventHandler<MapClickWinformsMapEventArgs>(mapMap_MapClick);
            this.btnConfirm.Click += new EventHandler(btnConfirm_Click);
            this.FormClosing += new FormClosingEventHandler(VerifyMap_FormClosing);
            btnZoomIn.Click+=new EventHandler(btnZoomIn_Click);
            btnZoomOut.Click+=new EventHandler(btnZoomOut_Click);
        }

        protected void VerifyMap_FormClosing(object sender, FormClosingEventArgs e)
        {
            UserEnteredLocation = txtLocation.Text;
        }

        protected void btnConfirm_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
        }

      
        #endregion

        #region Events

        /// <summary>
        /// Places an icon on the map for a clicked location. Also will reverse geocode the closest address.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void mapMap_MapClick(object sender, MapClickWinformsMapEventArgs e)
        {
            if (ClickCounter == 1)
            {
                ClickCounter = 0;
                return;
            }

            ClickCounter = 1;
            
            pbVerified.Visible = true;
            Verified = true;
            LayerOverlay markerOverlay = mapMap.Overlays["MarkerOverlay"] as LayerOverlay;
            MarkerLayer markerLayer = markerOverlay.Layers["MarkerLayer"] as MarkerLayer;
            markerLayer.MarkerLocation = new PointShape(e.WorldX, e.WorldY);

            // Reverse Geocode Location
            UsaGeocoder usaGeocoder = new UsaGeocoder(GeocoderPath);

            MatchResult matchResult = new MatchResult();
            try
            {
                usaGeocoder.Open();
                matchResult = usaGeocoder.Match(String.Format("{0} {1}", e.WorldX.ToString(), e.WorldY.ToString()));
            }
            catch (Exception ex)
            {
                MessageBox.Show(String.Format("RevGeocode error:\n{0}", ex.ToString()));
            }
            finally
            {
                usaGeocoder.Close();
            }

            if (matchResult.MatchItems.Count > 0)
            {
                MatchItem matchItem = matchResult.MatchItems[0];
                Address address = new Address(matchItem, new string[] { "HouseNumber", "PrefixDirectional", "Street", "StreetType", "SuffixDirectional", "City" });
                if (
                    MessageBox.Show(
                        String.Format("Closest address is {0}. Do you want to change your location text?", address.AddressText),
                        "Address Found", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1) == DialogResult.Yes
                    )
                {
                    txtLocation.Text = address.AddressText;
                }
            }

            //mapMap.Refresh(mapMap.Overlays["PointOverlay"]);
            mapMap.Refresh();
        }

        protected void btnZoomOut_Click(object sender, EventArgs e)
        {
            mapMap.ZoomOut(20);
            mapMap.Refresh();
        }

        protected void btnZoomIn_Click(object sender, EventArgs e)
        {
            mapMap.ZoomIn(60);
            mapMap.Refresh();
        }

        protected void txtLocation_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Return || e.KeyCode == Keys.Enter)
            {
                // Geocode address
                GeocodeLocation();
            }
        }

        protected void txtLocation_TextChanged(object sender, EventArgs e)
        {
            List<MatchResult> MatchResults = new List<MatchResult>();
            try
            {
                VerifyGeocoder.Open();
                //// This code was an attempt to limit the geocoding by Zip code, however it still returned results from all around the US.
                //foreach (string zip in CountyZips)
                //{
                //    MatchResults.Add(VerifyGeocoder.Match(txtLocation.Text, zip));
                //}
                MatchResults.Add(VerifyGeocoder.Match(txtLocation.Text));
                
            }
            catch (Exception ex)
            {
                MessageBox.Show(String.Format("Geocode error:\n{0}", ex.ToString()));
            }
            finally
            {
                VerifyGeocoder.Close();
            }

            FillAutoCompleteList(MatchResults);
        }

        protected void VerifyMap_Load(object sender, EventArgs e)
        {
            SetupMap();
            pbVerified.Visible = false;
            txtLocation.Text = UserEnteredLocation; // this is in relation to another form used for testing but not needed with this form.
        }

        #endregion

        #region Public Instance Properties

        public string CountyName { get; set; }
        public string UserEnteredLocation { get; set; }
        public bool Verified { get; set; }

        #endregion

        #region Private Instance Properties

        private UsaGeocoder VerifyGeocoder
        {
            get
            {
                if (_VerifyGeocoder == null)
                    _VerifyGeocoder = new UsaGeocoder(GeocoderPath, MatchStatus.FuzzyMatch);
                return _VerifyGeocoder;
            }
        }

        private List<Address> Locations
        {
            get
            {
                if (_Locations == null)
                    _Locations = new List<Address>();
                return _Locations;
            }
            set
            {
                _Locations = value;
            }
        }

        private AutoCompleteStringCollection AutoCompleteLocations
        {
            get
            {
                if (_AutoCompleteLocations == null)
                    _AutoCompleteLocations = new AutoCompleteStringCollection();
                return _AutoCompleteLocations;
            }
            set
            {
                _AutoCompleteLocations = value;
            }
        }

        private List<string> CountyZips 
        {
            get
            {
                if (_CountyZips == null)
                    _CountyZips = new List<string>();
                return _CountyZips;
            }
            set
            {
                _CountyZips = value;
            }
        }

        #endregion

        #region Private Instance Methods

        /// <summary>
        /// One of our goals is to have the location textbox include an autocomplete suggestion list as the user enters their address.
        /// </summary>
        /// <param name="MatchResults"></param>
        private void FillAutoCompleteList(List<MatchResult> MatchResults)
        {
            if (MatchResults.Count < 1)
            {
                return;
            }

            foreach (MatchResult match in MatchResults)
            {
                if (match.MatchItems.Count > 0)
                {
                    foreach (MatchItem matchItem in match.MatchItems)
                    {
                        Address temp = new Address(matchItem, new string[] { "HouseNumber", "PrefixDirectional", "Street", "StreetType", "SuffixDirectional", "City" });
                        if (!String.IsNullOrEmpty(temp.AddressText))
                        {
                            if (!Locations.Contains(temp))
                                Locations.Add(temp);
                            if (!AutoCompleteLocations.Contains(temp.AddressText))
                            {
                                Console.WriteLine(temp.AddressText);
                                AutoCompleteLocations.Add(temp.AddressText);
                            }
                        }
                    }
                }
            }

            
            txtLocation.AutoCompleteCustomSource = AutoCompleteLocations;
        }

        private void SetupMap()
        {
            mapMap.MapUnit = GeographyUnit.DecimalDegree;

            mapMap.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.Sand);

            //// Map marking of location is not involved with current test/issue
            //InMemoryFeatureLayer pointLayer = new InMemoryFeatureLayer();
            //pointLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.PointType = PointType.Bitmap;
            //pointLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.Image = new GeoImage(Path.Combine(mapbase, "Semi Truck.png"));
            //pointLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            ShapeFileFeatureLayer CountyLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039ctycu.shp"), ShapeFileReadWriteMode.ReadOnly);
            //ShapeFileFeatureLayer CountyLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Lake\IN089ctycu.shp"), ShapeFileReadWriteMode.ReadOnly);
            CountyLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.County1;
            CountyLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.StateBorder1;
            CountyLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            ShapeFileFeatureLayer CityLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039placecu.shp"), ShapeFileReadWriteMode.ReadOnly);
            //ShapeFileFeatureLayer CityLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Lake\IN089placecu.shp"), ShapeFileReadWriteMode.ReadOnly);
            CityLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Urban1;
            CityLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.City1("NAME");
            CityLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.StateBorder1;
            CityLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level15;
            CityLayer.ZoomLevelSet.ZoomLevel15.DefaultAreaStyle = AreaStyles.Urban1;
            CityLayer.ZoomLevelSet.ZoomLevel15.DefaultLineStyle = LineStyles.StateBorder1;
            CityLayer.ZoomLevelSet.ZoomLevel15.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            
            ShapeFileFeatureLayer HighwayLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039lkA.shp"), Path.Combine(INmapbase, @"Elkhart\IN039Highway.idx"), ShapeFileReadWriteMode.ReadOnly);
            //ShapeFileFeatureLayer HighwayLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Lake\IN089lkA.shp"), Path.Combine(INmapbase, @"Lake\IN089Highway.idx"), ShapeFileReadWriteMode.ReadOnly);
            HighwayLayer.ZoomLevelSet.ZoomLevel12.DefaultLineStyle = LineStyles.Highway3;
            HighwayLayer.ZoomLevelSet.ZoomLevel12.DefaultTextStyle = TextStyles.Highway2("FENAME");
            HighwayLayer.ZoomLevelSet.ZoomLevel12.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level14;
            HighwayLayer.ZoomLevelSet.ZoomLevel15.DefaultLineStyle = LineStyles.Highway3;
            HighwayLayer.ZoomLevelSet.ZoomLevel15.DefaultTextStyle = TextStyles.Highway2("FENAME");
            HighwayLayer.ZoomLevelSet.ZoomLevel15.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            ShapeFileFeatureLayer SecondaryLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039lkA.shp"), Path.Combine(INmapbase, @"Elkhart\IN039Secondary.idx"), ShapeFileReadWriteMode.ReadOnly);
            //ShapeFileFeatureLayer SecondaryLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Lake\IN089lkA.shp"), Path.Combine(INmapbase, @"Lake\IN089Secondary.idx"), ShapeFileReadWriteMode.ReadOnly);
            SecondaryLayer.ZoomLevelSet.ZoomLevel12.DefaultLineStyle = LineStyles.SecondaryRoad2;
            SecondaryLayer.ZoomLevelSet.ZoomLevel12.DefaultTextStyle = TextStyles.Highway2("FENAME");
            SecondaryLayer.ZoomLevelSet.ZoomLevel12.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level14;
            SecondaryLayer.ZoomLevelSet.ZoomLevel15.DefaultLineStyle = LineStyles.SecondaryRoad2;
            SecondaryLayer.ZoomLevelSet.ZoomLevel15.DefaultTextStyle = TextStyles.SecondaryRoad2("FENAME");
            SecondaryLayer.ZoomLevelSet.ZoomLevel15.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            
            //// Test after other layers of same file are open - This DOES work.
            //ShapeFileFeatureLayer AllRoads = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039lkA.shp"));
            //AllRoads.Open();
            //DataTable dt = AllRoads.QueryTools.ExecuteQuery("Select FENAME From IN039lkA Where FENAME Like('%SUBURBAN%');");
            //foreach (DataRow row in dt.Rows)
            //{
            //    Console.WriteLine(row["FENAME"]); // Just output to console for testing
            //}
            //AllRoads.Close();

            ShapeFileFeatureLayer RoadLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039lkA.shp"), Path.Combine(INmapbase, @"Elkhart\IN039LocalRoad.idx"), ShapeFileReadWriteMode.ReadOnly);
            //ShapeFileFeatureLayer RoadLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Lake\IN089lkA.shp"), Path.Combine(INmapbase, @"Lake\IN089LocalRoad.idx"), ShapeFileReadWriteMode.ReadOnly);
            RoadLayer.ZoomLevelSet.ZoomLevel12.DefaultLineStyle = LineStyles.LocalRoad4;
            RoadLayer.ZoomLevelSet.ZoomLevel12.DefaultTextStyle = TextStyles.LocalRoad3("FENAME");
            RoadLayer.ZoomLevelSet.ZoomLevel12.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level14;
            RoadLayer.ZoomLevelSet.ZoomLevel15.DefaultLineStyle = LineStyles.LocalRoad3;
            RoadLayer.ZoomLevelSet.ZoomLevel15.DefaultTextStyle = TextStyles.LocalRoad3("FENAME");
            RoadLayer.ZoomLevelSet.ZoomLevel15.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            ShapeFileFeatureLayer RailRoadLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039lkB.shp"), ShapeFileReadWriteMode.ReadOnly);
            //ShapeFileFeatureLayer RailRoadLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Lake\IN089lkB.shp"), ShapeFileReadWriteMode.ReadOnly);
            RailRoadLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.Railway2;
            RailRoadLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            ShapeFileFeatureLayer WaterLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039wat.shp"), ShapeFileReadWriteMode.ReadOnly);
            //ShapeFileFeatureLayer WaterLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Lake\IN089wat.shp"), ShapeFileReadWriteMode.ReadOnly);
            WaterLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Water1;
            WaterLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.Water1("LANDNAME");
            WaterLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            InMemoryFeatureLayer IntersectionLayer = new InMemoryFeatureLayer();
            IntersectionLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.Highway5;
            IntersectionLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Circle, GeoColor.StandardColors.Red, 12);
            IntersectionLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;


            LayerOverlay MapLayerOverlay = new LayerOverlay();
            MapLayerOverlay.Layers.Add("COUNTY", CountyLayer);
            MapLayerOverlay.Layers.Add("CITY", CityLayer);
            MapLayerOverlay.Layers.Add("LOCALROAD", RoadLayer);
            MapLayerOverlay.Layers.Add("HIGHWAY", HighwayLayer);
            MapLayerOverlay.Layers.Add("SECONDARY", SecondaryLayer);
            MapLayerOverlay.Layers.Add("RAIL", RailRoadLayer);
            MapLayerOverlay.Layers.Add("WATER", WaterLayer);
            MapLayerOverlay.Layers.Add("INTERSECTION", IntersectionLayer);

            mapMap.Overlays.Add("MapOverlay", MapLayerOverlay);

            //LayerOverlay pointOverlay = new LayerOverlay();
            //pointOverlay.Layers.Add("PointLayer", pointLayer);
            //mapMap.Overlays.Add("PointOverlay", pointOverlay);

            // Marker Overlay
            LayerOverlay markerOverlay = new LayerOverlay();
            mapMap.Overlays.Add("MarkerOverlay", markerOverlay);

            // Marker layer
            MarkerLayer markerLayer = new MarkerLayer();
            markerOverlay.Layers.Add("MarkerLayer", markerLayer);

            ScaleLineAdornmentLayer scaleLine = new ScaleLineAdornmentLayer();
            scaleLine.Location = AdornmentLocation.LowerLeft;
            scaleLine.XOffsetInPixel = 25f;
            scaleLine.YOffsetInPixel = -25f;

            mapMap.AdornmentOverlay.Layers.Add("ScaleLineAdornmentLayer", scaleLine);
            
            // Attempt to filter geocode by specific ZIPs. It still searched outside the ZIP code.
            //RoadLayer.Open();
            //DataTable dt = RoadLayer.QueryTools.ExecuteQuery("select distinct ZIPL from IN039lkA");
            //foreach (DataRow row in dt.Rows)
            //{
            //    CountyZips.Add(row["ZIPL"].ToString());
            //}
            //RoadLayer.Close();
            
            mapMap.MapClick += new EventHandler<MapClickWinformsMapEventArgs>(mapMap_MapClick);

            SetMaxView();
        }

        private void SetMaxView()
        {
            LayerOverlay MapLayer = mapMap.Overlays["MapOverlay"] as LayerOverlay;
            ShapeFileFeatureLayer CountyLayer = MapLayer.Layers["COUNTY"] as ShapeFileFeatureLayer;
            CountyLayer.Open();
            mapMap.CurrentExtent = CountyLayer.GetBoundingBox();
            CountyLayer.Close();
            mapMap.Refresh();
        }

        private void GeocodeLocation()
        {
            MatchResult matchResult = new MatchResult();
            if (txtLocation.Text.Contains("&"))
            {
                string[] streets = txtLocation.Text.Split('&');
                if (streets.Length != 2)
                    return;

                string street1 = streets[0].Trim();
                string street2 = streets[1].Trim();
                
                LayerOverlay MapOverlay = mapMap.Overlays["MapOverlay"] as LayerOverlay;
                ShapeFileFeatureLayer StreetLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039lkA.shp"), ShapeFileReadWriteMode.ReadOnly);
                //ShapeFileFeatureLayer StreetLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Lake\IN089lkA.shp"), ShapeFileReadWriteMode.ReadOnly);

                //// --------------------------------------------------------------------------
                //// This is where I was attempting to use the QueryTools and get the error. 
                //StreetLayer.Open();
                //string cst1 = ConvertStreet(street1);
                //string cst2 = ConvertStreet(street2);
                //DataTable dt1 = StreetLayer.QueryTools.ExecuteQuery("Select FENAME From IN039lkA Where FENAME Like('%" + street1 + "%');");
                //DataTable dt2 = StreetLayer.QueryTools.ExecuteQuery("Select FENAME From IN039lkA Where FENAME Like('%" + street2 + "%');");
                //StreetLayer.Close();
                //Collection<Feature> street1features = new Collection<Feature>();
                //Collection<Feature> street2features = new Collection<Feature>();
                //foreach (DataRow row in dt1.Rows)
                //{
                //    Console.WriteLine(row["FENAME"]);
                //    Collection<Feature> temp = StreetLayer.FeatureSource.GetFeaturesByColumnValue("FENAME", (string)row["FENAME"]);
                //    foreach (Feature t in temp)
                //        street1features.Add(t);
                //}
                //foreach (DataRow row in dt2.Rows)
                //{
                //    Console.WriteLine(row["FENAME"]);
                //    Collection<Feature> temp = StreetLayer.FeatureSource.GetFeaturesByColumnValue("FENAME", (string)row["FENAME"]);
                //    foreach (Feature t in temp)
                //        street2features.Add(t);
                //}
                //// --------------------------------------------------------------------------

                // --------------------------------------------------------------------------
                // This is the working code that I have been using in my testing
                StreetLayer.Open();
                Collection<Feature> street1features = StreetLayer.FeatureSource.GetFeaturesByColumnValue("FENAME", ConvertStreet(street1));//"County Road 17");
                Collection<Feature> street2features = StreetLayer.FeatureSource.GetFeaturesByColumnValue("FENAME", ConvertStreet(street2));//"Suburban");
                StreetLayer.Close();
                // --------------------------------------------------------------------------

                InMemoryFeatureLayer IntersectionLayer = MapOverlay.Layers["INTERSECTION"] as InMemoryFeatureLayer;
                IntersectionLayer.Open();
                IntersectionLayer.EditTools.BeginTransaction();
                Collection<MultilineShape> street1shapes = new Collection<MultilineShape>();
                Collection<MultilineShape> street2shapes = new Collection<MultilineShape>();

                foreach (Feature st1 in street1features)
                {
                    MultilineShape multiLineShape = st1.GetShape() as MultilineShape;
                    street1shapes.Add(multiLineShape);
                    //IntersectionLayer.EditTools.Add(new Feature(multiLineShape));
                }
                foreach (Feature st2 in street2features)
                {
                    MultilineShape multiLineShape = st2.GetShape() as MultilineShape;
                    street2shapes.Add(multiLineShape);
                    //IntersectionLayer.EditTools.Add(new Feature(multiLineShape));
                }

                IntersectionLayer.InternalFeatures.Clear();
                bool IntersectionFound = false;
                try
                {
                    IntersectionLayer.EditTools.Add(new Feature(GetIntersectionPoints(street1shapes, street2shapes, 50, DistanceUnit.Meter, GeographyUnit.DecimalDegree)));
                    IntersectionFound = true;
                }
                catch (Exception ex)
                {
                    MessageBox.Show("No intersection found");
                    IntersectionFound = false;
                }
                
                IntersectionLayer.EditTools.CommitTransaction();

                // --------------------------------------------------------------------------
                // This is where I was attempting to 'zoom in' on the intersection. 
                // The problem is that it doesn't work as expected, i.e. I have a single dot but it won't center and zoom.
                // When I have 2 intersections, as in "Jackson & Main" in Elkhart County, Indiana, it will change the extent 
                // based on both of them as I would want

                // This didn't work for what I was looking for at all.
                //mapMap.CurrentExtent = ExtentHelper.GetBoundingBoxOfItems(IntersectionLayer.InternalFeatures);
                if (IntersectionFound)
                {
                    if (IntersectionLayer.InternalFeatures.Count == 1)
                        // This also didn't work for me, as there is only one feature that may be both dots.
                        //mapMap.ZoomIntoCenter(95, IntersectionLayer.InternalFeatures.First());
                        
                        // This works ok for when I have 2 intersections.
                        mapMap.CurrentExtent = IntersectionLayer.InternalFeatures[0].GetShape().GetBoundingBox();
                    else
                        SetMaxView();
                }
                else
                    SetMaxView();
                    
                IntersectionLayer.Close();

                mapMap.Refresh();
                return;
            }

            // This is code I was using to geocode a standard address entered. It works fine and is pretty quick
            // provided data is formatted properly. I did have problems however limiting the matching to certain Zip codes only.
            try
            {
                VerifyGeocoder.Open();
                matchResult = VerifyGeocoder.Match(txtLocation.Text);
            }
            finally
            {
                VerifyGeocoder.Close();
            }

            if (matchResult.MatchItems.Count < 1)
            {
                MessageBox.Show("Address not found, please select a location on the map.");
                return;
            }
            MatchItem matchItem = matchResult.MatchItems[0];
            LayerOverlay markerOverlay = mapMap.Overlays["MarkerOverlay"] as LayerOverlay;
            MarkerLayer markerLayer = markerOverlay.Layers["MarkerLayer"] as MarkerLayer;
            markerLayer.MarkerLocation = new PointShape(matchItem.MatchPairs["CentroidPoint"]);

            // Set the extent around the address and refresh the map
            mapMap.CurrentExtent = new RectangleShape(matchItem.MatchPairs["BoundingBox"]);
            mapMap.Refresh();
        }

        //Gets the intersection point(s) based on the distance tolerance from the end points of the two collection of MultiLineShapes.
        //We use a tolerance because we want to have an intersection being counted as such even if the points don't have the exact X and Y values.
        private MultipointShape GetIntersectionPoints(Collection<MultilineShape> multiLineShapes1, Collection<MultilineShape> multiLineShapes2, double toleranceDistance, DistanceUnit toleranceUnit, GeographyUnit mapUnit)
        {
            MultipointShape intersectionMultiPointShape = new MultipointShape();

            MultipointShape multiPointShape1 = GetEndPoints(multiLineShapes1);
            MultipointShape multiPointShape2 = GetEndPoints(multiLineShapes2);

            foreach (PointShape pointShape1 in multiPointShape1.Points)
            {
                foreach (PointShape pointShape2 in multiPointShape2.Points)
                {
                    if (pointShape1.GetDistanceTo(pointShape2, mapUnit, toleranceUnit) <= toleranceDistance)
                    {
                        intersectionMultiPointShape.Points.Add(pointShape1);
                    }
                }
            }
            return intersectionMultiPointShape;
        }

        private MultipointShape GetEndPoints(Collection<MultilineShape> multilineShapes)
        {
            MultipointShape multipointShape = new MultipointShape();
            foreach (MultilineShape multilineShape in multilineShapes)
            {
                foreach (LineShape lineShape in multilineShape.Lines)
                {
                    multipointShape.Points.Add(new PointShape(lineShape.Vertices[0]));
                    multipointShape.Points.Add(new PointShape(lineShape.Vertices[lineShape.Vertices.Count - 1]));
                }
            }
            return multipointShape;
        }

        /// <summary>
        /// Simple conversions for Primary and Secondary road abbreviations.
        /// </summary>
        /// <param name="Street"></param>
        /// <returns></returns>
        private string ConvertStreet(string Street)
        {
            if (String.IsNullOrEmpty(Street))
                return Street;
            string temp = Street.ToUpper();
            if (temp.Contains("SR"))
                return Regex.Replace(temp, "SR(?<num>\\d[0-9]+)", @"State Road ${num}");
            if (temp.Contains("CR"))
                return Regex.Replace(temp, "CR(?<num>\\d[0-9]+)", @"County Road ${num}");
            if (temp.Contains("US"))
                return Regex.Replace(temp, "US(?<num>\\d[0-9]+)", @"United States Highway ${num}");

            return Street;
        }

        #endregion
    }
}
